@@ -5,7 +5,7 @@ use warnings;
 package Test::Modern;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.007';
+our $VERSION   = '0.009';
 
 use Cwd              0     qw();
 use Exporter::Tiny   0.030 qw();
@@ -138,6 +138,52 @@ $HINTS{ lib } = sub { $_[3]{lib}++; () };
 	};
 }
 
+{
+	my $checker;
+	$checker = sub {
+		$checker = eval q{
+			package #
+				Dummy::Test::RequiresInternet;
+			use Socket;
+			sub {
+				my ($host, $port) = @_;
+				my $portnum = ($port =~ /[^0-9]/) ? getservbyname($port, "tcp") : $port;
+				die "Could not find a port number for $port\n" if not $portnum;
+				my $iaddr = inet_aton($host) or die "no host: $host\n";
+				my $paddr = sockaddr_in($portnum, $iaddr);
+				my $proto = getprotobyname("tcp");
+				socket(my $sock, PF_INET, SOCK_STREAM, $proto) or die "socket: $!\n";
+				connect($sock, $paddr) or die "connect: $!\n";
+				close($sock) or die "close: $!\n";
+				!!1;
+			}
+		};
+		goto($checker);
+	};
+	
+	$HINTS{ internet } = sub
+	{
+		plan skip_all => 'Not running network tests'
+			if $ENV{NO_NETWORK_TESTING};
+		
+		my $arg = $_[2];
+		my @hosts =
+			ref($arg) eq q(HASH)   ? %$arg :
+			ref($arg) eq q(ARRAY)  ? @$arg :
+			ref($arg) eq q(SCALAR) ? ( $$arg => 80 ) :
+			( 'www.google.com' => 80 );
+		
+		while (@hosts) {
+			my ($host, $port) = splice(@hosts, 0, 2);
+			defined($host) && defined($port)
+				or BAIL_OUT("Expected host+port pair (not undef)");
+			eval { $checker->($host, $port); 1 } or plan skip_all => "$@";
+		}
+		
+		return;
+	};
+}
+
 our @ISA = qw(Exporter::Tiny);
 our %EXPORT_TAGS = (
 	more     => [qw(
@@ -357,7 +403,7 @@ sub object_ok
 	my $name   = (@_%2) ? shift : '$object';
 	my %tests  = @_;
 	
-	my $result = subtest("$name ok", sub
+	my $result = &subtest("$name ok", sub
 	{
 		if (ref($object) eq q(CODE))
 		{
@@ -413,7 +459,7 @@ sub object_ok
 		if (exists($tests{more}))
 		{
 			my $more = delete $tests{more};
-			subtest("more tests for $name", sub
+			&subtest("more tests for $name", sub
 			{
 				my $exception = exception { $object->$more };
 				is($exception, undef, "no exception thrown by additional tests");
@@ -459,40 +505,77 @@ sub Test::Modern::_TD::AUTOLOAD
 	{
 		my $ns = shift;
 		require_module($ns);
-
+		
 		my %symbols = do {
 			no strict qw(refs);
-			map   { /(\w+)$/ => $_; }
-			grep  { *$_{CODE}; }
+			map   { /([^:]+)$/; $1 => $_; }
+			grep  { eval { *$_{CODE} } }
 			values %{"$ns\::"};
 		};
 		
+		my @imports;
 		my $meta;
 		if ($INC{ module_notional_filename('Moose::Util') }
 			and $meta = Moose::Util::find_meta($ns))
 		{
 			my %subs = %symbols;
 			delete @subs{ $meta->get_method_list };
-			return keys %subs;
+			@imports = keys %subs;
 		}
 		elsif ($INC{ module_notional_filename('Mouse::Util') }
 			and $meta = Mouse::Util::class_of($ns))
 		{
 			my %subs = %symbols;
 			delete @subs{ $meta->get_method_list };
-			return keys %subs;
+			@imports = keys %subs;
 		}
 		else
 		{
 			require B;
 			no strict qw(refs);
-			return grep {
+			@imports = grep {
 				my $stash = B::svref_2object(\&{"$ns\::$_"})->GV->STASH->NAME;
 				$stash ne $ns
 					and $stash ne 'Role::Tiny'
 					and not eval { require Role::Tiny; Role::Tiny->is_role($stash) }
 			} keys %symbols;
 		}
+		
+		my %imports; @imports{@imports} = map substr("$symbols{$_}", 1), @imports;
+		
+		# But really it's better to inherit these rather than import them. :-)
+		if (keys %imports) {
+			delete @imports{qw(import unimport)};
+		}
+		
+		if (keys %imports) {
+			my @overloads = grep {
+				/^\(/ or $imports{$_} eq 'overload::nil'
+			} keys %imports;
+			delete @imports{@overloads} if @overloads;
+		}
+		
+		if (keys %imports and $] < 5.010) {
+			my @constants = grep { $imports{$_} eq 'constant::__ANON__' } keys %imports;
+			delete @imports{@constants} if @constants;
+		}
+		
+		@imports = sort keys(%imports);
+		my %sources;
+		@sources{@imports} = map {
+			B::svref_2object(\&{"$ns\::$_"})->GV->STASH->NAME;
+		} @imports;
+		
+		my %does;
+		for my $func (keys %sources) {
+			my $role = $sources{$func};
+			$does{$role} = !!eval { $ns->DOES($role) }
+				unless exists $does{$role};
+			delete $imports{$func}
+				if $does{$role};
+		}
+		
+		sort keys(%imports);
 	};
 	
 	my $_diag_dirt = sub
@@ -508,7 +591,7 @@ sub Test::Modern::_TD::AUTOLOAD
 		} @imports;
 		diag explain('remaining imports: ' => \%imports);
 	};
-
+	
 	my $_test_or_skip = sub
 	{
 		my $ns = shift;
@@ -526,7 +609,7 @@ sub Test::Modern::_TD::AUTOLOAD
 		};
 		return $rv;
 	};
-		
+	
 	sub namespaces_clean
 	{
 		local $Test::Builder::Level = $Test::Builder::Level + 1;
@@ -535,7 +618,7 @@ sub Test::Modern::_TD::AUTOLOAD
 		return shift->$_test_or_skip if @_ == 1;
 		
 		my @namespaces = @_;
-		return subtest(
+		return &subtest(
 			sprintf("namespaces_clean: %s", join q(, ), @namespaces),
 			sub {
 				$_->$_test_or_skip for @namespaces;
@@ -616,7 +699,7 @@ sub Test::Modern::_TD::AUTOLOAD
 		
 		local $Test::Builder::Level = $Test::Builder::Level + 1;
 		
-		subtest $name => sub
+		&subtest($name => sub
 		{
 			my %versions;
 			for my $file (@files)
@@ -633,7 +716,7 @@ sub Test::Modern::_TD::AUTOLOAD
 					for sort keys(%versions);
 			}
 			done_testing;
-		};
+		});
 	};
 	
 	_wrap("Test::Pod", "pod_file_ok", extended => 1);
@@ -1019,6 +1102,23 @@ For example:
 
 =back
 
+=head2 Features inspired by Test::RequiresInternet
+
+Similarly you can skip the test script if an Internet connection is not
+available:
+
+   use Test::Modern -internet;
+
+You can check for the ability to connect to particular hosts and ports:
+
+   use Test::Modern -internet => [
+      'www.example.com'  => 'http',
+      '8.8.8.8'          => 53,
+   ];
+
+Test::Modern does not use L<Test::RequiresInternet> but I've stolen much
+of the latter's implementation.
+
 =head2 Features inspired by Test::Without::Module
 
 Test::Modern does not use L<Test::Without::Module>, but does provide
@@ -1221,6 +1321,10 @@ C<shouldnt_warn>.
 
 Classify the test script.
 
+=item C<< -internet >>
+
+The test script requires Internet access.
+
 =item C<< -requires >>, C<< -without >>
 
 Specify modules required or hidden for these test cases.
@@ -1250,6 +1354,12 @@ and L</"Features from Test::Version">.
 They also can trigger certain import tags to skip a test script. See
 L</"Features inspired by Test::DescribeMe">.
 
+=item C<NO_NETWORK_TESTS>
+
+Automatically skips any tests which indicate that they require Internet
+access, without even checking to see if the Internet is accessible.
+See L</"Features inspired by Test::RequiresInternet">.
+
 =item C<PERL_TEST_MODERN_ALLOW_WARNINGS>
 
 Setting this to true allows you to disable L<Test::Warnings>' end test.
@@ -1282,6 +1392,7 @@ L<Test::Moose>,
 L<Test::CleanNamespaces>,
 L<Test::Requires>,
 L<Test::Without::Module>,
+L<Test::RequiresInternet>,
 L<Test::DescribeMe>,
 L<Test::Lib>,
 L<Test::Pod>,
